@@ -22,21 +22,10 @@
 #include <libnetfilter_conntrack/libnetfilter_conntrack_l3extensions.h>
 #include <libnetfilter_conntrack/libnetfilter_conntrack_extensions.h>
 
+#include "internal.h"
+
 #define NFCT_BUFSIZE 4096
 
-typedef int (*nfct_handler)(struct nfct_handle *cth, struct nlmsghdr *nlh,
-			    void *arg);
-
-/* Harald says: "better for encapsulation" ;) */
-struct nfct_handle {
-	struct nfnl_handle *nfnlh;
-	struct nfnl_subsys_handle *nfnlssh_ct;
-	struct nfnl_subsys_handle *nfnlssh_exp;
-	nfct_callback callback;		/* user callback */
-	void *callback_data;		/* user data for callback */
-	nfct_handler handler;		/* netlink handler */
-};
-
 static char *lib_dir = LIBNETFILTER_CONNTRACK_DIR;
 static LIST_HEAD(proto_list);
 static LIST_HEAD(l3proto_list);
@@ -151,6 +140,14 @@ int nfct_close(struct nfct_handle *cth)
 		cth->nfnlssh_ct = NULL;
 	}
 
+	/* required by the new API */
+	cth->cb = NULL;
+	free(cth->nfnl_cb.data);
+
+	cth->nfnl_cb.call = NULL; 
+	cth->nfnl_cb.data = NULL;
+	cth->nfnl_cb.attr_count = 0;
+
 	err = nfnl_close(cth->nfnlh);
 	free(cth);
 
@@ -479,11 +476,11 @@ static void nfct_parse_counters(struct nfattr *attr,
 					NFA_DATA(tb[CTA_COUNTERS_BYTES-1]));
 	if (tb[CTA_COUNTERS32_PACKETS-1])
 		ct->counters[dir].packets
-			= htonl(*(u_int32_t *)
+			= ntohl(*(u_int32_t *)
 				NFA_DATA(tb[CTA_COUNTERS32_PACKETS-1]));
 	if (tb[CTA_COUNTERS32_BYTES-1])
 		ct->counters[dir].bytes
-			= htonl(*(u_int32_t *)
+			= ntohl(*(u_int32_t *)
 				NFA_DATA(tb[CTA_COUNTERS32_BYTES-1]));
 }
 
@@ -1125,12 +1122,6 @@ int nfct_dump_conntrack_table_reset_counters(struct nfct_handle *cth,
 
 int nfct_event_conntrack(struct nfct_handle *cth)
 {
-	/*
-	 * You need to be root to listen to conntrack events
-	 */
-	if (getuid() != 0)
-		return -EPERM;
-	
 	cth->handler = nfct_conntrack_netlink_handler;
 	return nfnl_listen(cth->nfnlh, &callback_handler, cth);
 }
@@ -1298,12 +1289,6 @@ int nfct_delete_expectation(struct nfct_handle *cth, struct nfct_tuple *tuple,
 
 int nfct_event_expectation(struct nfct_handle *cth)
 {
-	/*
-	 * You need to be root to listen to conntrack events
-	 */
-	if (getuid() != 0)
-		return -EPERM;
-
 	cth->handler = nfct_expect_netlink_handler;
 	return nfnl_listen(cth->nfnlh, &callback_handler, cth);
 }
